@@ -5,15 +5,36 @@
 JMX::Jmx4Perl - Access to JMX via Perl
 
 =head1 SYNOPSIS
+
+Simple:
+
+   use strict;
+   use JMX::Jmx4Perl;
+   use JMX::Jmx4Perl::Alias;   # Import certains aliases for MBeans
+
+   print "Memory Used: ",
+          JMX::Jmx4Perl
+              ->new(url => "http://localhost:808get0/j4p-agent")
+              ->get_attribute(MEMORY_HEAP_USED);
+
+Advanced:
+
+   use strict;
+   use JMX::Jmx4Perl;
+   use JMX::Jmx4Perl::Request;   # Type constants are exported here
    
-   my $jmx = new JMX::Jmx4Perl(url => "http://localhost:8080/j4p-agent");
-   my $request = new JMX::Jmx4Perl::Request(type => READ_ATTRIBUTE,
+   my $jmx = new JMX::Jmx4Perl(url => "http://localhost:8080/j4p-agent",
+                               product => "jboss");
+   my $request = new JMX::Jmx4Perl::Request(type => READ,
                                             mbean => "java.lang:type=Memory",
                                             attribute => "HeapMemoryUsage",
                                             path => "used");
    my $response = $jmx->request($request);
    print "Memory used: ",$response->value(),"\n";
 
+   # Get general server information
+   print "Server Info: ",$jmx->info();
+
 =head1 DESCRIPTION
 
 Jmx4Perl is here to connect the Java and Perl Enterprise world by providing
@@ -80,10 +101,11 @@ package JMX::Jmx4Perl;
 use Carp;
 use JMX::Jmx4Perl::Request;
 use strict;
-use vars qw($VERSION);
+use vars qw($VERSION $HANDLER_BASE_PACKAGE @PRODUCT_HANDLER_ORDERING);
 use Data::Dumper;
+use Module::Find;
 
-$VERSION = "0.1";
+$VERSION = "0.15";
 
 my $REGISTRY = {
                 # Agent based
@@ -92,15 +114,53 @@ my $REGISTRY = {
                 "JJAgent" => "JMX::Jmx4Perl::Agent",
                };
 
-=item $jmx = JMX::Jmx4Perl->new(mode => <access module>, ....)
+my %PRODUCT_HANDLER;
+
+sub _register_handlers { 
+    my $handler_package = shift;
+    %PRODUCT_HANDLER = ();
+    
+    my @id2order = ();
+    for my $handler (findsubmod $handler_package) {
+        next unless $handler;
+        my $handler_file = $handler;
+        $handler_file =~ s|::|/|g;
+        require $handler_file.".pm";
+        next if $handler eq $handler_package."::BaseHandler";
+        my $id = eval "${handler}::id()";
+        croak "No id() method on $handler: $@" if $@;
+        $PRODUCT_HANDLER{lc $id} = $handler;
+        push @id2order, [ lc $id, $handler->order() ];
+    }
+    # Ordering Schema according to $handler->order():
+    # -10,-5,-3,0,undef,undef,undef,1,8,9,1000
+    my @high = map { $_->[0] } sort { $a->[1] <=> $b->[1] } grep { defined($_->[1]) && $_->[1] <= 0 } @id2order;
+    my @med  = map { $_->[0] } grep { not defined($_->[1]) } @id2order;
+    my @low  = map { $_->[0] } sort { $a->[1] <=> $b->[1] } grep { defined($_->[1]) && $_->[1] > 0 } @id2order;
+    @PRODUCT_HANDLER_ORDERING = (@high,@med,@low);
+}
+
+BEGIN {
+    &_register_handlers("JMX::Jmx4Perl::Product");
+}
+
+
+=item $jmx = JMX::Jmx4Perl->new(mode => <access module>, product => <id>, ....)
 
 Create a new instance. The call is dispatched to an Jmx4Perl implementation by
 selecting an appropriate mode. For now, the only mode supported is "agent",
 which uses the L<JMX::Jmx4Perl::Agent> backend. Hence, the mode can be
 submitted for now.
 
-Any other named parameters are interpreted by the backend, please refer to its
-documentation for details (i.e. L<JMX::Jmx4Perl::Agent>)
+If you provide a product id via the named parameter C<product> you can given
+B<jmx4perl> a hint which server you are using. By default, this module uses
+autodetection to guess the kind of server you are talking to. You need to
+provide this argument only if you use B<jmx4perl>'s alias feature and if you
+want to speed up things (autodetection can be quite slow since this requires
+several JMX request to detect product specific MBean attributes).
+
+Any other named parameters are interpreted by the backend, please
+refer to its documentation for details (i.e. L<JMX::Jmx4Perl::Agent>)
 
 =cut
 
@@ -109,12 +169,19 @@ sub new {
     my $cfg = ref($_[0]) eq "HASH" ? $_[0] : {  @_ };
     
     my $mode = delete $cfg->{mode} || &autodiscover_mode();
+    my $product = $cfg->{product} ? lc delete $cfg->{product} : undef;
+
     $class = $REGISTRY->{$mode} || croak "Unknown runtime mode " . $mode;
+    if ($product && !$PRODUCT_HANDLER{lc $product}) {
+        croak "No handler for product '$product'. Known Handlers are [".(join ", ",keys %PRODUCT_HANDLER)."]";
+    }
+
     eval "require $class";
     croak "Cannot load $class: $@" if $@;
 
     my $self = { 
                 cfg => $cfg,
+                product => $product
              };
     bless $self,(ref($class) || $class);
     $self->init();
@@ -126,10 +193,15 @@ sub new {
 =item $resp => $jmx->get_attribute(...)
 
   $value = $jmx->get_attribute($mbean,$attribute,$path) 
+  $value = $jmx->get_attribute($alias)
+  $value = $jmx->get_attribute(ALIAS)       # Literal alias as defined in
+                                            # JMX::Jmx4Perl::Alias
   $value = $jmx->get_attribute({ domain => <domain>, 
-                                properties => { <key> => value }, 
-                                attribute => <attribute>, 
-                                path => <path>)
+                                 properties => { <key> => value }, 
+                                 attribute => <attribute>, 
+                                 path => <path> })
+  $value = $jmx->get_attribute({ alias => <alias>, 
+                                 path => <path })
 
 Read a JMX attribute. In the first form, you provide the MBean name, the
 attribute name and an optional path as positional arguments. The second
@@ -142,36 +214,187 @@ MBeans please refer to
 L<http://java.sun.com/j2se/1.5.0/docs/api/javax/management/ObjectName.html> for
 more information about JMX naming.
 
-This method returns the value as it is returned from the server
+Alternatively, you can provide an alias, which gets resolved to its real name
+by so called I<product handler>. Several product handlers are provided out of
+the box. If you have specified a C<product> id during construction of this
+object, the associated handler is selected. Otherwise, autodetection is used to
+guess the product. Note, that autodetection is potentially slow since it
+involves several JMX calls to the server. If you call with a single, scalar
+value, this argument is taken as alias (without any path). If you want to use
+aliases together with a path, you need to use the second form with a hash ref
+for providing the (named) arguments. 
+
+This method returns the value as it is returned from the server.
 
 =cut 
 
 sub get_attribute {
     my $self = shift;
-    my ($object,$attribute,$path);
+    my ($object,$attribute,$path,$alias_path);
     if (ref($_[0]) eq "HASH") {
-        $object = $_[0]->{mbean};
-        if (!$object && $_[0]->{domain} && ($_[0]->{properties} || $_[0]->{props})) {
-            $object = $_[0]->{domain} . ":";
-            my $href = $_[0]->{properties} || $_[0]->{props};
-            croak "'properties' is not a hashref" unless ref($href);
-            for my $k (keys %{$href}) {
-                $object .= $k . "=" . $href->{$k};
+        if ($_[0]->{alias}) {
+            ($object,$attribute,$alias_path) = $self->resolve_attribute_alias($_[0]->{alias});
+            if (ref($object) eq "CODE") {
+                # Let the handler do it
+                return $self->request_attribute_by_handler($object);
             }
+            if ($alias_path) {
+                $path = $_[0]->{path} ? $_[0]->{path} . "/" . $alias_path : $alias_path;
+            } else { 
+                $path = $_[0]->{path};
+            }
+        } else {
+            $object = $_[0]->{mbean};
+            if (!$object && $_[0]->{domain} && ($_[0]->{properties} || $_[0]->{props})) {
+                $object = $_[0]->{domain} . ":";
+                my $href = $_[0]->{properties} || $_[0]->{props};
+                croak "'properties' is not a hashref" unless ref($href);
+                for my $k (keys %{$href}) {
+                    $object .= $k . "=" . $href->{$k};
+                }
+            }
+            $attribute = $_[0]->{attribute};
+            $path = $_[0]->{path};
         }
-        $attribute = $_[0]->{attribute};
-        $path = $_[0]->{path};
+        
     } else {
-        ($object,$attribute,$path) = @_;
+        if (@_ == 1) {
+            # A single argument can only be used as an alias
+            ($object,$attribute,$path) = 
+              $self->resolve_attribute_alias($_[0]);
+            if (ref($object) eq "CODE") {
+                # Let the handler do it
+                return $self->request_attribute_by_handler($object);
+            }
+        } else {
+            ($object,$attribute,$path) = @_;
+        }
     }
     croak "No object name provided" unless $object;
     croak "No attribute provided for object $object" unless $attribute;
 
-    my $request = JMX::Jmx4Perl::Request->new(READ_ATTRIBUTE,$object,$attribute,$path);
+    my $request = JMX::Jmx4Perl::Request->new(READ,$object,$attribute,$path);
     my $response = $self->request($request);
     return $response->value;
 }
 
+=item $resp = $jmx->request($request)
+
+Send a request to the underlying agent and return the response. This is an
+abstract method which needs to be overwritten by a subclass. The argument must
+be of type L<JMX::Jmx4Perl::Request> and it returns an object of type
+L<JMX::Jmx4Perl::Response>.
+
+=cut 
+
+sub request {
+    croak "Internal: Must be overwritten by a subclass";    
+}
+
+
+=item $info = $jmx->info($verbose)
+
+Get a textual description of the server as returned by a product specific
+handler (see L<JMX::Jmx4Perl::Product::BaseHandler>). It uses the
+autodetection facility if no product is given explicitely during construction. 
+
+If C<$verbose> is true, print even more information
+
+=cut
+
+sub info {
+    my $self = shift;
+    my $verbose = shift;
+    my $handler = $self->{product_handler} || $self->_create_handler();
+    return $handler->info($verbose);
+}
+
+
+=item ($object,$attribute,$path) = $self->resolve_attribute_alias($alias)
+
+Resolve an alias for an attibute. This is done by querying registered product
+handlers for resolving an alias. This method will croak if a handler could be
+found but not such alias is known by C<jmx4perl>. 
+
+If the C<product> was not set during construction, the first call to this
+method will try to autodetect the server. If it cannot determine the proper
+server it will throw an exception. 
+
+Returns the object, attribute, path triple which can be used for requesting the
+server or C<undef> if the handler can not resolve this alias.
+
+A handler can decide to handle the fetching of the alias value directly. In
+this case, this metod returns the code reference which needs to be executed
+with the handler as argument (see "request_attribute_by_handler") below. 
+
+=cut
+
+sub resolve_attribute_alias {
+    my $self = shift;
+    my $alias = shift || croak "No alias provided";
+
+    my $handler = $self->{product_handler} || $self->_create_handler();    
+    return $handler->attribute_alias($alias);
+}
+
+=item $attribute_value = $self->request_attribute_by_handler($coderef)
+
+Execute a subroutine with the current handler as argument and returns the
+return value of this subroutine. This method is used in conjunction with
+C<resolve_attribute_alias> to allow handler a more sophisticated way to access
+the MBeanServer. Additionally, it allows for mangling the output, too.
+
+=cut
+
+sub request_attribute_by_handler {
+    my $self = shift;
+    my $code = shift;
+    my $handler = $self->{product_handler} || $self->_create_handler();    
+    return &{$code}($handler);
+}
+
+=item $product = $self->product_id()
+
+For supported application servers, this methods returns the name of the
+product. This product is either detected automatically or provided during
+construction time.
+
+=cut 
+
+sub product {
+    my $self = shift;
+    my $handler = $self->{product_handler} || $self->_create_handler();
+    return $handler->name();
+}
+
+sub _create_handler {
+    my $self = shift;
+    $self->{product} ||= $self->_autodetect_product();
+    croak "Cannot autodetect server product" unless $self->{product};
+    
+    $self->{product_handler} = $self->_new_handler($self->{product});
+    return $self->{product_handler};        
+}
+
+sub _autodetect_product {
+    my $self = shift;
+    for my $id (@PRODUCT_HANDLER_ORDERING) {
+        my $handler = $self->_new_handler($id);
+        return $id if $handler->autodetect();
+    }
+    return undef;
+}
+
+sub _new_handler {
+    my $self = shift;
+    my $product = shift;
+
+    my $handler = eval $PRODUCT_HANDLER{$product}."->new(\$self)";
+    croak "Cannot create handler ",$self->{product},": $@" if $@;
+    return $handler;
+}
+
+
 =item $value = $jmx->list($path)
 
 Get all MBeans as registered at the specified server. A C<$path> can be
@@ -208,10 +431,12 @@ given, the returned value has the following format
               .... 
            };
 
-A complete path has the format C<"<domain>/<property
-list>/("attribute"|"operation")/<index>"> (e.g. C<java.lang/name=Code
-Cache,type=MemoryPool/attribute/0>). A path can be provided partially, in which
-case the remaining map/array is returned.
+A complete path has the format C<"E<lt>domainE<gt>/E<lt>property
+listE<gt>/("attribute"|"operation")/E<lt>indexE<gt>">
+(e.g. C<java.lang/name=Code Cache,type=MemoryPool/attribute/0>). A path can be
+provided partially, in which case the remaining map/array is returned. See also
+L<JMX::Jmx4Perl::Agent::Protocol> for a more detailed discussion of inner
+pathes. 
 
 =cut
 
@@ -219,7 +444,7 @@ sub list {
     my $self = shift;
     my $path = shift;
 
-    my $request = JMX::Jmx4Perl::Request->new(LIST_MBEANS,$path);
+    my $request = JMX::Jmx4Perl::Request->new(LIST,$path);
     my $response = $self->request($request);
     return $response->value;    
 }
@@ -232,8 +457,8 @@ sub list {
 Get the a formatted string representing the MBeans as returnded by C<list()>.
 C<$path> is the optional inner path for selecting only a subset of all mbean.
 See C<list()> for more details. If called with a L<JMX::Jmx4Perl::Response>
-object, the list will be taken from the provided response object and not
-fetched from the server
+object, the list and the optional path will be taken from the provided response
+object and not fetched again from the server.
 
 =cut
 
@@ -261,6 +486,8 @@ sub formatted_list {
 
 my $SPACE = 4;
 my @SEPS = (":");
+my $CURRENT_DOMAIN = "";
+
 sub _format_map { 
     my ($ret,$map,$path,$level) = @_;
     
@@ -275,7 +502,13 @@ sub _format_map {
         $ret = &_format_map($ret,$map,$path,$level);
     } else {
         for my $d (keys %$map) {
-            $ret .= &_get_space($level).$d.$sep."\n" unless ($d eq "attr" || $d eq "op");
+            my $prefix = "";
+            if ($level == 0) {
+                $CURRENT_DOMAIN = $d;
+            } elsif ($level == 1) {
+                $prefix = $CURRENT_DOMAIN . ":";
+            } 
+            $ret .= &_get_space($level).$prefix.$d.$sep."\n" unless ($d eq "attr" || $d eq "op");
             my @args = ($ret,$map->{$d},$path);
             if ($d eq "attr") {
                 $ret = &_format_attr_or_op(@args,$level,"attr","Attributes",\&_format_attribute);
@@ -348,19 +581,6 @@ sub _get_space {
     return " " x ($level * $SPACE);
 }
 
-=item $resp = $jmx->request($request)
-
-Send a request to the underlying agent and return the response. This is an
-abstract method which needs to be overwritten by a subclass. The argument must
-be of type L<JMX::Jmx4Perl::Request> and it returns an object of type
-L<JMX::Jmx4Perl::Response>.
-
-=cut 
-
-sub request {
-    croak "Internal: Must be overwritten by a subclass";    
-}
-
 sub cfg {
     my $self = shift;
     my $key = shift;
@@ -392,30 +612,6 @@ sub autodiscover_mode {
 
 =back
 
-=head1 ROADMAP
-
-=over
-
-=item * 
-
-Suport for writing attributes 
-
-=item * 
-
-Support for executing JMX operations
-
-=item *
-
-Providing aliases for common MBean Attributes, which can be used for any
-application server
-
-=item *
-
-JSR-160 access to a remote JMX Mbean-Server, which requires integration of a
-JVM (with something like L<Java::Import>)
-
-=back
-
 =head1 LICENSE
 
 This file is part of jmx4perl.
