@@ -2,24 +2,28 @@
 package JMX::Jmx4Perl::Agent;
 
 use JSON;
-use LWP::UserAgent;
+use URI::Escape;
 use HTTP::Request;
 use Carp;
 use strict;
 use vars qw($VERSION $DEBUG @ISA);
 use JMX::Jmx4Perl;
 use JMX::Jmx4Perl::Request;
+use JMX::Jmx4Perl::Response;
 use JMX::Jmx4Perl::Agent::UserAgent;
+use Data::Dumper;
 
 @ISA = qw(JMX::Jmx4Perl);
 
+$VERSION = $JMX::Jmx4Perl::VERSION;
+
 =head1 NAME 
 
-JMX::Jmx4Perl::Agent - JSON-HTTP based acess to a remote JMX Agent
+JMX::Jmx4Perl::Agent - JSON-HTTP based acess to a remote JMX agent
 
 =head1 SYNOPSIS
 
- my $agent = new JMX::Jmx4Perl(mode=>"agent", url => "http://jeeserver/jjagent");
+ my $agent = new JMX::Jmx4Perl(mode=>"agent", url => "http://jeeserver/j4p-agent");
  my $answer = $agent->get_attribute("java.lang:type=Memory","HeapMemoryUsage");
  print Dumper($answer);
 
@@ -30,6 +34,7 @@ JMX::Jmx4Perl::Agent - JSON-HTTP based acess to a remote JMX Agent
                 'max' => 532742144,
                 'init' => 0
                },
+    'status' => 200,
     'request' => {
                    'attribute' => 'HeapMemoryUsage',
                    'name' => 'java.lang:type=Memory'
@@ -56,8 +61,8 @@ Creates a new local agent for a given url
 
 The url where the agent is deployed. This is a mandatory parameter. The url
 must include the context within the server, which is typically based on the
-name of the war archive. Example: http://localhost:8080/jjagent for a drop in
-deployment of the agent in a standard Tomcat's webapp directory. 
+name of the war archive. Example: C<http://localhost:8080/j4p-agent> for a drop
+in deployment of the agent in a standard Tomcat's webapp directory.
 
 =item timeout => <timeout>
 
@@ -93,7 +98,7 @@ sub init {
     $ua->jjagent_config($self->{cfg});
     $ua->timeout($self->cfg-('timeout')) if $self->cfg('timeout');
     $ua->agent("JMX::Jmx4Perl::Agent $VERSION");
-    $ua->env_proxy;
+    # $ua->env_proxy;
     my $proxy = $self->cfg('proxy');
     if ($proxy) {
         if (ref($proxy) eq "HASH") {
@@ -112,7 +117,7 @@ sub init {
 =item $resp = $agent->request($request)
 
 Implementation of the JMX request as specified in L<JMX::Jmx4Perl>. It uses a
-L<HTTP:Request> sent via an L<LWP::UserAgent> for posting a JSON representation
+L<HTTP::Request> sent via an L<LWP::UserAgent> for posting a JSON representation
 of the request. This method shouldn't be called directly but via
 L<JMX::Jmx4Perl>->request(). 
 
@@ -126,7 +131,20 @@ sub request {
     my $url = $self->request_url($jmx_request);
     my $req = HTTP::Request->new(GET => $url);
     my $resp = $ua->request($req);
-    my $ret = from_json($resp->content());
+    my $ret;
+    eval {
+        $ret = from_json($resp->content());
+    };
+    if ($@) {
+        return new JMX::Jmx4Perl::Response->new
+          ( 
+           $resp->code,
+           $jmx_request,
+           $resp->content,
+           "Error while deserializing JSON answer (probably wrong URL)"
+          );
+    }
+    croak "Error while deserializing ",$resp->content," : ",$@ if $@;
     if ($resp->is_error && !$ret->{status}) {
         my $error = "Error while fetching $url :\n" . $resp->status_line . "\n";
         my $content = $resp->content;
@@ -135,9 +153,7 @@ sub request {
             $error .=  $content if $content ne $resp->status_line;
         }
         croak $error;
-    }
-    
-
+    };
     return JMX::Jmx4Perl::Response->new($ret->{status},$jmx_request,$ret->{value},$ret->{error},$ret->{stacktrace});
 }
 
@@ -154,19 +170,33 @@ sub request_url {
     $url .= "/" unless $url =~ m|/$|;
     my $type = $request->get("type");
     $url .= $type . "/";
-    $url .= $request->get("mbean") . "/";
-    if ($type eq READ_ATTRIBUTE || $type eq WRITE_ATTRIBUTE) {
-        $url .= $request->get("attribute");
-        $url .= "/" . $request->get("path") if $request->get("path");
-        if ($type eq WRITE_ATTRIBUTE) {
-            $url .= "/" . $request->get("value");
+    $url .= $self->_escape($request->get("mbean")) . "/";
+    if ($type eq READ || $type eq WRITE) {
+        $url .= $self->_escape($request->get("attribute"));
+        $url .= "/" . $self->_escape($request->get("path")) if $request->get("path");
+        if ($type eq WRITE) {
+            $url .= "/" . $self->_escape($request->get("value"));
         }
-    } elsif ($type eq LIST_MBEANS) {
-        $url .= $request->get("path") if $request->get("path");
+    } elsif ($type eq LIST) {
+        $url .= $self->_escape($request->get("path")) if $request->get("path");
     }
     return $url;
 }
 
+# Escape '/' which are used as separators by using "/-/" as an escape sequence
+# URI Encoding doesn't work for slashes, since some Appserver tend to mangle
+# them up with pathinfo-slashes to early in the request cycle.
+# E.g. Tomcat/Jboss croaks with a "HTTP/1.x 400 Invalid URI: noSlash" if one
+# uses an encoded slash somewhere in the path-info part.
+sub _escape {
+    my $self = shift;
+    my $input = shift;
+    $input =~ s|(/+)|"/" . ('-' x length($1)) . "/"|eg;    
+    return uri_escape_utf8($input,"^A-Za-z0-9\-_.!~*'()/");   # Added "/" to
+                                                              # default
+                                                              # set. See L<URI>
+}
+
 =back
 
 =cut 
