@@ -9,12 +9,12 @@ use Carp qw(croak);
 
 =head1 NAME
 
-JMX::Jmx4Perl::Product::Jetty - Handler for Jetty 
+JMX::Jmx4Perl::Product::Jetty - Handler for Jetty (http://www.mortbay.org/jetty/)
 
 =head1 DESCRIPTION
 
-This is the product handler support Jetty. It supports Jetty version 5 and 6.
-(L<http://www.mortbay.org/jetty/>)
+This is the product handler support Jetty. It supports Jetty version 5, 6 and
+7.
 
 Please note, that you must have JMX support enabled in Jetty for autodetection
 and aliasing to work. See the Jetty documentation for details.
@@ -31,33 +31,11 @@ sub name {
 
 sub _try_version {
     my $self = shift;
-    my $jmx = $self->{jmx4perl};
-
-    # Jetty V6 & 7
-    my $servers = $jmx->search("*:id=0,type=server,*");
-    my $ret;
-    if ($servers) {
-        $ret = $self->try_attribute("version",$servers->[0],"version");
-    }
-
-    # Jetty V5
-    if (!length($self->{version})) {
-        delete $self->{version};
-        $ret = $self->try_attribute("version","org.mortbay:jetty=default","version");
-    }
-
+    my $ret = $self->try_attribute("version","org.mortbay:jetty=default","version");
     $self->{version} =~ s/Jetty\/([^\s]+).*/$1/;
     return $ret;
 }
 
-sub autodetect_pattern {
-    return "version";
-}
-
-sub vendor {
-    return "Mortbay";
-} 
-
 sub jsr77 {
     return 0;
 }
